#  Copyright Qantom software private limited - all rights reserved
# No part of this code may be edited or used without the written 
# permission of Qantom's management. 

from WiNoThing import WiThing

__author__ = "raghu"
__date__ = "$April 20, 2018 00:15:57 PM$"

class WiKeypad(WiThing):
  
  def __init__(self, name, host): 
    print "before"; 
    WiThing.__init__(self, name, host);
   
  def keys(self):
    return WiThing.invoke_rest(self, "getKeys") ;


# Copyright Qantom software private limited - all rights reserved
# No part of this code may be edited or used without the written
# permission of Qantom's management.

__author__ = "raghu"
__date__ = "$13 Apr, 2018 5:19:56 PM$"


import urllib
import time



from winotwrappers.WiNoThing import WiNoThing
from winotwrappers.WiNoTSwitch import WiNoTSwitch


from winotwrappers.DistanceMeter import WinotDistanceMeter
from winotwrappers.WiLed import WiLed
from winotwrappers.WiBuzzer import WiBuzzer 
from winotwrappers.WiLdr import WiLdr
from winotwrappers.MotionSensor import WiMotionSensor
from winotwrappers.WiPushButton import WiPushButton

if __name__ == "__main__":
  
  #try:
    host="pluggabletrainer.thingsping.in"; 
    
    pb = WiPushButton(host)
    print ("Ispushed = " + str(pb.pushed())); 
    
    pir1 = WiMotionSensor(host); 
    print ("Motion = " + str(pir1.is_there_motion())); 
    
    ldr1 = WiLdr(host); 
    print (str(ldr1.intensity()))
    
    led1 = WiLed(host); 
    led1.on() ; 
    print "Turned on!"
    time.sleep(0.5); 
    led1.off() ; 
    print "Turned off"; 
    
    buzz1 = WiBuzzer(host); 
    buzz1.buzz(500)
    
  
    #scale1 = WinotDistanceMeter("pluggabletrainer.thingsping.in");
    #print "Distance = " + str(scale1.distance()); 
    
    '''
    switch1 = WiNoTSwitch("s1", "wemostest1.thingsping.in");
    switch1.switch() ; 
    nm = switch1.get_name(); 
    print "Switch - " + nm + " is " + switch1.get_status() ; 
    
    motionsensor = WinotMotionSensor.create_sensor("192.168.0.205", "staircasemotion", 13);
    floorswitch = WiNoTSwitch.create_switch("192.168.0.205", "hightrigger", "staircase", 12);
    
    if (motion.sensor.is_there_motion()):
      floorswitch.switch_on() ; 
    else :
      floorswitch.switch_off() ; 
    '''
    
  #except BaseException as e:
  #  print "Error trying to process request - " + e.message





#  Copyright Qantom software private limited - all rights reserved
# No part of this code may be edited or used without the written 
# permission of Qantom's management. 

import urllib
import socket

__author__ = "raghu"
__date__ = "$April 20, 2018 00:15:57 PM$"

class WinotDisplay:
  def __init__(self, name, host=0): 
    self._name = name
    if (host == 0) :
      domainname = self._name  ; 
      try :
        self._address = "http://" + socket.gethostbyname(domainname)
      except : 
        raise BaseException("Could not determine address for " + name)
    else :
      self._address = "http://" + host
      
  def display(self, line1, line2=""):
    fulladdr = self._address + "/displayText?line1=" +  line1 + "&line2=" + line2
    urllib.urlopen(fulladdr)
      
  def lighton(self):
    fulladdr = self._address + "/displayText?backlight=on"
    print fulladdr
    urllib.urlopen(fulladdr)
    
  def lightoff(self):
    fulladdr = self._address + "/displayText?backlight=off"
    urllib.urlopen(fulladdr)



#  Copyright Qantom software private limited - all rights reserved
# No part of this code may be edited or used without the written
# permission of Qantom's management.

import urllib
import socket
import time

__author__ = "raghu"
__date__ = "$April 20, 2018 00:15:57 PM$"
## Look up https://github.com/dbader/schedule

WIRARP_PORT = 9219
ARP_STALETIME = 60*.1 # Max time for which cached values should be used.
ARP_TIMEOUT = 5
WINOTARPREQ = "WiNoTWho"
WINOTARPFORALL = "WiNoTAll" 

class WiThing:

  # Create Dictionary with keys set to device names and 
  # value set to a set of device IP with epochtime
  devtable = {} # Format {dev_name : {dev_ip, epochtime}}

  def __init__(self, host, name=None):
    # type: (object, object) -> object
    self._name = name;
    self._address = "http://" + host;


  @staticmethod
  def get_all_winotdevices() :
    #First of all clear the device table dictionary.
    #as we will be finding the values fresh!
    WiThing.devtable.clear();
    t = int(time.time());
    arpmsg = WINOTARPREQ + "\n"
    arpmsg = arpmsg + WINOTARPFORALL + "\n"
    arpmsg = arpmsg + str(t) + "\n"

    clientSock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    clientSock.bind(('', 0))
    clientSock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    clientSock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
    clientSock.sendto(arpmsg, ('<broadcast>', WIRARP_PORT))

    try:
      # declare our serverSocket upon which
      # we will be listening for UDP messages
      serverSock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

      # One difference is that we will have to bind our declared IP address
      # and port number to our newly declared serverSock
      serverSock.bind(("0.0.0.0", WIRARP_PORT))
      serverSock.settimeout(ARP_TIMEOUT)
      while True:
        data, addr = serverSock.recvfrom(1024)
        #print "Recd. data = " + data
        recdArp = data.split("\n")
        if (recdArp[0] == "WiNoTMe"):
          recdName = recdArp[1]
          recdUid = recdArp[2]
          if (recdUid == str(t)):
            recdIp = recdArp[3]
            WiThing.devtable[recdName] = [recdIp, t]
          else:
            print ("Recd response doesn't match with sent request" + data)
        else:
          print ("Unknown response received - " + data)
      serverSock.settimeout(None)
    except Exception as e:
      pass # In this case, if there is an exception doesn'tnecessarily mean an error.

    retList = {}
    for devname in WiThing.devtable.keys():
      tmpVal = WiThing.devtable[devname]
      retList[devname] =  tmpVal[0]
    return retList

  @staticmethod
  def resolve_address(device_name):
    t = int(time.time());
    toResolve = True ;
    recdIp = ""
    if device_name in WiThing.devtable:
      storetime = WiThing.devtable[device_name][1]
      if ((t - storetime) > ARP_STALETIME):
        toResolve = True
      else :
        recdIp = WiThing.devtable[device_name][0]
        print("Using cached value : " + recdIp)
        toResolve = False

    if(toResolve == True):
      arpmsg = WINOTARPREQ + "\n"
      arpmsg = arpmsg + device_name + "\n"
      arpmsg = arpmsg + str(t) + "\n"
      # No need to send IP address

      clientSock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
      clientSock.bind(('', 0))
      clientSock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
      clientSock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
      clientSock.sendto(arpmsg, ('<broadcast>', WIRARP_PORT))

      try:
        # declare our serverSocket upon which
        # we will be listening for UDP messages
        serverSock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

        # One difference is that we will have to bind our declared IP address
        # and port number to our newly declared serverSock
        serverSock.bind(("", WIRARP_PORT))
        serverSock.settimeout(ARP_TIMEOUT)
        data, addr = serverSock.recvfrom(1024)
        serverSock.settimeout(None)
        recdArp = data.split("\n")
        if (recdArp[0] == "WiNoTMe") :
          recdUid = recdArp[1]
          if (recdUid == str(t)) :
            recdIp = recdArp[2]
            WiThing.devtable[device_name] = [recdIp, t]
          else :
            print ("Recd response doesn't match with sent request" + data)
        else:
          print ("Unknown response received - " + data)
      except Exception as e:
        print("Could not resolve device address for " + device_name)
        recdIp = None ;
    return recdIp

  def get_name(self) :
    return self._name ;

  def invoke_rest(self, command, additionalparams=None):
    fullcommand = "/" + str(command) ;
    try :
      if (self._name is not None) :
        fullcommand = fullcommand + "?thingname=" + self._name ;
        if(additionalparams):
          fullcommand = fullcommand + "&" + additionalparams ;
      elif (additionalparams):
        fullcommand = fullcommand + "?" + additionalparams ;

      #print "Invoking " + self._address + fullcommand ;
      response = urllib.urlopen(self._address + fullcommand) ;

    except Exception as e:
      raise BaseException("Could not connect to WiNoT device @ " + self._address + fullcommand);
    return response.read().strip();

  def set_name(self, newname):
    result = self.invoke_rest("setName", "&name=" + newname) ;
    if (result.startswith("FAIL")):
      result = result[5:] ;
      raise BaseException("Error while trying to set name. Message = " + result);
    self._name = newname ;

  def get_all_commands(self):
    response = self.invoke_rest("getAllCommands");
    name_command_pairs = response.split(",")
    cmd_dict = {}
    for pair in name_command_pairs:
      ncsplit =  pair.split(":")
      cmd_dict[ncsplit[0].strip()] = ncsplit[1].strip()
    return cmd_dict

  def analog_read(self):
    response = self.invoke_rest("analogRead")
    return int(response) 

  def digital_read(self, pin):
    response = self.invoke_rest("digitalRead", "pin=" + str(pin))
    return int(response)

  def digital_write(self, pin, state):
    response = self.invoke_rest("digitalWrite", "pin=" +str(pin) + "&state=" +str(state))


name = "winot"

## The following imports are required because it will allow us to
## directly refer to the Class names in our python scripts instead of
## having to refer to individual file names.
## For example we can now say
##   from winot import *
##   led = WiLed(addr)
## Without the below import statements, we will have to say
##  import winot
##  from winot.WiBuzzer import WiBuzzer
##  from winot.WiLed import WiLed
##  and so on

from WiBuzzer import WiBuzzer
from WiKeypad import WiKeypad
from WiLdr import WiLdr
from WiLed import WiLed
from WiMotionSensor import WiMotionSensor
from WiNoThing import WiThing
from WiNoTSwitch import WiSwitch
from WiPushButton import WiPushButton
from WiScale import WiScale
from WiWeather import WiWeather
from RemoteWeather import * 


#  Copyright Qantom software private limited - all rights reserved
# No part of this code may be edited or used without the written 
# permission of Qantom's management. 

from WiNoThing import WiThing

__author__ = "raghu"
__date__ = "$April 20, 2018 00:15:57 PM$"


class WiMotionSensor(WiThing):

    def __init__(self, host, name=None):
      WiThing.__init__(self, host, name);

    def is_there_motion(self):
        # type: () -> bool
        response = WiThing.invoke_rest(self, "checkForMotion");
        is_motion = False
        if (response == "Motion Detected"):
            is_motion = True
        else:
            is_motion = False
        return is_motion



#  Copyright Qantom software private limited - all rights reserved
# No part of this code may be edited or used without the written
# permission of Qantom's management.

from WiNoThing import WiThing

__author__ = "raghu"
__date__ = "$February 11th 2019"


class WiWeather(WiThing):

    def __init__(self, host, name=None):
        WiThing.__init__(self, host, name);

    def temperature(self):
        response = WiThing.invoke_rest(self, "weather", "read=temperature");
        if (response.startswith("T=")) :
            return float(response.split("=")[1]);
        else:
            return None ;

    def humidity(self):
        response = WiThing.invoke_rest(self, "weather", "read=humidity");
        response = response.strip()
        if (response.startswith("H=")) :
            return float(response.split("=")[1]);
        else:
            return None ;

    def weather(self):
        response = WiThing.invoke_rest(self, "weather", "read=both");
        response = response.strip()
        if (response.startswith("T=")) :
            tmpArr = response.split("=")
            t=tmpArr[1].strip("+H")
            h=tmpArr[2].strip()
            return (float(t), float(h));
        else:
            return None ;


#  Copyright Qantom software private limited - all rights reserved
# No part of this code may be edited or used without the written
# permission of Qantom's management.

__author__ = "raghu"
__date__ = "$April 20, 2018 00:15:57 PM$"

from WiNoThing import WiThing

class WiSwitch(WiThing):

  def __init__(self, host, name=None):
    WiThing.__init__(self, host, name);
    #super(name, host).__init__();

  @classmethod
  def create_switch(self, address, type, name, pin):
    wiswitch = WiSwitch(name, address);
    if (type.lower() == "lowtrigger" or type.lower() == "low") :
      WiSwitch.invoke_rest("addThing", "type=lowtrigger&pin=" +pin)
    else :
      WiSwitch.invoke_rest("addThing", "type=hightrigger&pin=" +pin)
    return wiswitch ;

  def switch(self):
    WiThing.invoke_rest(self, "Switch", "action=switch") ;

  def on(self):
    WiThing.invoke_rest(self, "Switch", "action=on") ;

  def off(self):
    WiThing.invoke_rest(self, "Switch", "action=off") ;

  def status(self):
    return WiThing.invoke_rest(self, "Switch");




import sys
import time
import socket

from WiNoThing import WiNoThing
from WiLed import WiLed

'''
for ip in socket.gethostbyname_ex(socket.gethostname()):
  #print (socket.gethostbyname(ip))
  print ip
  #print ("[" + ip + "]" )

exit() ;
'''

addr = WiNoThing.resolve_address("nodemcu2") ;
if (addr != None) :
  print("Address = " + addr);
addr = WiNoThing.resolve_address("nodemcu2") ;
if (addr != None) :
  print("Address = " + addr);
addr = WiNoThing.resolve_address("nodemcu23") ;
if (addr != None) :
  print("Address = " + addr);
time.sleep(10)
print("----- Trying after cache time -----")
addr = WiNoThing.resolve_address("nodemcu2") ;
if (addr != None) :
  print("Address = " + addr);

if (addr is None) :
  exit() ;

led = WiLed(addr);

def TurnOnLed():
  led.on();

def TurnOffLed():
  led.off();

TurnOnLed() ;
time.sleep(2);
TurnOffLed() ;


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="winot",
    version="0.8.6",
    author="Qantom Software Private Limited",
    author_email="info@qantom.com",
    description="WiNoT Wrappers",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://www.qantom.com/winot_trainer.html",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
)


#  Copyright Qantom software private limited - all rights reserved
# No part of this code may be edited or used without the written 
# permission of Qantom's management. 

from WiNoThing import WiThing

__author__ = "raghu"
__date__ = "$April 20, 2018 00:15:57 PM$"

class WiLed(WiThing):
  
  def __init__(self, host, name=None): 
    WiThing.__init__(self,host,name);
   
  def on(self):
    response =  WiThing.invoke_rest(self, "operateLed",additionalparams="action=ON") ;
    return response;
    
  def off(self):
    response = WiThing.invoke_rest(self, "operateLed",additionalparams="action=OFF") ;
    return response;

  def switch(self):
    response =  WiThing.invoke_rest(self, "operateLed",additionalparams="action=SWITCH") ;
    return response;

  def blink(self, btime):
    response = WiThing.invoke_rest(self, "operateLed",
          additionalparams="action=blink&blinkTime=" + str(btime)) ;


# -*- coding: utf-8 -*-
import sys, getopt

from WiNoThing import WiThing
from WiScale import WiScale
from WiBuzzer import WiBuzzer
from WiLdr import WiLdr
from WiPushButton import WiPushButton
from WiNoTSwitch import WiSwitch
from WiWeather import WiWeather
from WiMotionSensor import WiMotionSensor
from WiLed import WiLed
from WiBuzzer import WiBuzzer

def test_all() :
    devices = WiThing.get_all_winotdevices()

    if (len(devices) == 0):
        print ("No WiNoT Devices found!")
    else:
        cnt = 0;
        sel_device = 1;  # type: int
        for device in devices.keys():
            cnt = cnt + 1
        if (cnt > 1):
            idx = 1;
            for device in devices.keys():
                print ("%r. Device name = %-15s| Address = %s" % (idx, device.strip(), devices[device]))
                idx = idx + 1
            sel_device = int(raw_input("Choose a device from the above list(0 to exit) - "))
        if (sel_device == 0):
            exit(0)
        else:
            sel_device = sel_device - 1;
            dev_name = devices.keys()[sel_device];
            dev_ip = devices[dev_name];
            thing = WiThing(dev_ip)
            cmds = thing.get_all_commands().values()

            if (len(cmds) == 8 and  cmds.__contains__("buzz") and cmds.__contains__("getDistance")
                    and cmds.__contains__("checkForMotion") and cmds.__contains__("operateLed")
                    and cmds.__contains__("Switch") and cmds.__contains__("isPushed")
                    and cmds.__contains__("intensity") and cmds.__contains__("weather")):
                # It means that this is a trainer.

                # print "Selected device is " + dev_name + " whose address is " + dev_ip
                # print ("%s %s(%s) %" ("", dev_name, dev_ip, "------------------"))
                print ("\nRunning trainer tests for %s(%s)" % (dev_name, dev_ip))

                ms = WiMotionSensor(dev_ip)
                isMotion = ms.is_there_motion();
                weather = WiWeather(dev_ip)
                temperature = weather.temperature();
                humidity = weather.humidity();
                distance = WiScale(dev_ip).distance();
                intensity = WiLdr(dev_ip).intensity();
                WiBuzzer(dev_ip).buzz();
                WiLed(dev_ip).switch();
                WiSwitch(dev_ip).switch();

                print ("Test Completed. If you observed the following then " \
                       "your WiNoT® device is working fine.  \n" \
                       "1. The buzzer should have beeped for 3 seconds \n" \
                       "2. The LED status should have changed \n" \
                       "3. The Relay status should have changed (You would have heard a small click)\n\n" \
                       "Values from onboard sensors:")

                print ("%-35s %r" % ("Has motion been detected?", isMotion))
                print ("%-35s %r %s" % ("Distance from nearest object", distance, "cms"))
                print ("%-35s %r %s" % ("Ambient temperature", temperature, "degree C"))
                print ("%-35s %r %s" % ("Ambient humidity", humidity, "%"))
                print ("%-35s %r" % ("Ambient light intensity level", intensity))
            else:
                print ("Device " + dev_name, " has these things - ", cmds)
            # print sel_dev_commands

def show_devices() :
    devices = WiThing.get_all_winotdevices()

    if (len(devices) == 0):
        print ("No WiNoT Devices found!")
    else:
        cnt = 0;
        sel_device = 1;  # type: int
        for device in devices.keys():
            cnt = cnt + 1
            print ("%r. Device name = %-15s| Address = %s" % (cnt, device.strip(), devices[device]))


if __name__ == '__main__':
    if (len(sys.argv) == 1 ):
        print "Usage : python -m winot <option>\n" \
            "Where options can be one of :\n" \
            "test -  Test a WiNoT board\n" \
            "list -  List all the WiNoT devices in your network\n"
    else :
        arg = sys.argv[1]
        if (arg == "test"):
            test_all()
        elif (arg == "list") :
            show_devices()





import urllib
import json

# "https://api.openweathermap.org/data/2.5/weather?q=Bengaluru&appid=1a0622b746356b944847f0e1432e741c&units=metric"

def get_weather_data(city):
    url = "https://api.openweathermap.org/data/2.5/weather?q="
    url = url + city + "&appid="
    with open("wkey.txt", "r") as fh:
        owmkey = fh.read().strip()
    url = url + owmkey
    url = url + "&units=metric"
    try :
        response = urllib.urlopen(url)
        
        response = json.loads(response.read().strip())
        response = response["main"]
        temp = response["temp"]
        humidity = response["humidity"]
        response = (temp, humidity)
    except Exception as e:
        print "Could not connect to remote weather"
        response = None
    
    return response

class RemoteWeather:
    @staticmethod
    def get_internet_weather(city) :
        return get_weather_data(city)